﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data.Entity;

namespace PersonalFinance
{
    /// <summary>
    /// Create DB context for entities
    /// </summary>
    public class EntityContext : DbContext
    {
        public DbSet<Payment> Payments { get; set; }
        public DbSet<ScheduledItem> ScheduledItems { get; set; }
        public DbSet<User> Users { get; set; }
    }

    /// <summary>
    /// Initialization of Conxtext (temporary Alway, in production will IfModelChanges)
    /// </summary>
    public class EntityContextInit : DropCreateDatabaseAlways<EntityContext>
    {
        protected override void Seed(EntityContext context)
        {
            //chyba se týkala toho, že uživatel má povinný parametr login...
            //dá se zjistit přes context.GetValidationErrors()

            List<User> userList = GetInitUserList();
            context.Users.AddRange(userList);

            List<Payment> paymentList = GetInitPaymentList(userList);
            context.Payments.AddRange(paymentList);

            List<ScheduledItem> scheduledList = GetInitScheduledItemList(paymentList);
            context.ScheduledItems.AddRange(scheduledList);

            context.SaveChanges();
        }

        /// <summary>
        /// Predefined users (used for Moq too)
        /// </summary>
        /// <returns>List of Usesr</returns>
        public List<User> GetInitUserList()
        {
            return new List<User>(){
                new User() { UserId = 1, Login = "Tonda", Name = "Antonín", Lastname = "Neumann", Sex = Sex.Male},
                new User() { UserId = 2, Login = "Ivetka", Name = "Iveta", Lastname = "Dobrá", Sex = Sex.Female },
                new User() { UserId = 3, Login = "Honzík", Name = "Jan", Lastname = "Novák", Sex = Sex.Male },
                new User() { UserId = 4, Login = "Andy", Name = "Andrea", Lastname = "Nováková", Sex = Sex.Female },
                new User() { UserId = 5, Login = "ĚŠČŘŽÝÁÍÉÚŮěščřžýáíéúů", Name = "John", Lastname = "Doe", Sex = Sex.Male }
            };
        }

        /// <summary>
        /// Predefined ScheduledItems (used for Moq too)
        /// </summary>
        /// <param name="paymentList">List of Payments</param>
        /// <returns>List of ScheduledItems</returns>
        public List<ScheduledItem> GetInitScheduledItemList(List<Payment> paymentList)
        {
            Payment[] payment = paymentList.ToArray();
            return new List<ScheduledItem>() { 
                new ScheduledItem() { ScheduledItemId= 1, Frequency = Frequency.Monthly, LastUse = DateTime.Now, Payment = payment[0] },
                new ScheduledItem() { ScheduledItemId= 2, Frequency = Frequency.Monthly, LastUse = DateTime.Now, Payment = payment[3] },
                new ScheduledItem() { ScheduledItemId= 3, Frequency = Frequency.Quarterly, LastUse = DateTime.Now, Payment = payment[4] }
            };
        }

        /// <summary>
        /// Predefined Payments (used for Moq too)
        /// </summary>
        /// <param name="userList">List of users</param>
        /// <returns>List of Paments</returns>
        public List<Payment> GetInitPaymentList(List<User> userList)
        {
            User[] user = userList.ToArray();
            return new List<Payment>() {
                new Payment(user[0])//0
                {
                    PaymentId = 1,
                    Currency = PaymentCurrency.CZK,
                    Date = DateTime.Now,
                    Price = 1700,
                    Purpose = "Elektřina",
                    Group = PaymentGroup.Family,
                    Paid = true,
                    Tags = new List<string> { "CEZ", "elektrika", "test" }
                },
                new Payment(user[0])//1
                {
                    PaymentId = 2,
                    Currency = PaymentCurrency.EUR,
                    Date = DateTime.Now,
                    Price = 85,
                    Purpose = "Telefon",
                    Group = PaymentGroup.Personal,
                    Paid = false,
                    Tags = new List<string> { "mobil", "T-Com", "test" }
                },
                new Payment(user[0])//2
                {
                    PaymentId = 3,
                    Currency = PaymentCurrency.USD,
                    Date = DateTime.Now,
                    Price = 150,
                    Purpose = "Dárek",
                    Note = "Dárek by měl dorazit XX.YY.",
                    Group = PaymentGroup.Private,
                    Paid = true,
                    Tags = new List<string> { "ebay", "darek", "test" }
                },
                new Payment(user[1])//3
                {
                    PaymentId = 4,
                    Currency = PaymentCurrency.CZK,
                    Date = DateTime.Now,
                    Price = 2200,
                    Purpose = "Elektřina",
                    Group = PaymentGroup.Family,
                    Paid = true,
                    Tags = new List<string> { "CEZ", "elektrika", "test" }
                },
                new Payment(user[1])//4
                {
                    PaymentId = 5,
                    Currency = PaymentCurrency.CZK,
                    Date = DateTime.Now,
                    Price = 1200,
                    Purpose = "Povinné ručení",
                    Group = PaymentGroup.Personal,
                    Paid = true,
                    Tags = new List<string> { "auto", "pojisteni", "test" }
                }
            };
        }
    }
}
