﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PersonalFinance
{
    /// <summary>
    /// Service for communication to database
    /// </summary>
    public class PaymentService
    {
        private EntityContext _context;
        public PaymentService(EntityContext context)
        {
            _context = context;
        }

        /// <summary>
        /// Add new payment
        /// </summary>
        /// <param name="p"></param>
        /// <returns>Added payment</returns>
        public Payment AddPayment(Payment p)
        {
            _context.Payments.Add(p);
            _context.SaveChanges();
            return p;
        }

        /// <summary>
        /// Create and add new payment
        /// </summary>
        /// <param name="price"></param>
        /// <param name="currency"></param>
        /// <param name="date"></param>
        /// <param name="purpose"></param>
        /// <param name="paid"></param>
        /// <param name="group"></param>
        /// <returns>Added payment</returns>
        public Payment AddPayment(int price, PaymentCurrency currency, DateTime date, string purpose, bool paid, PaymentGroup group)
        {
            User u = new User() { Login = "Test" };
            return AddPayment(price, currency, date, purpose, paid, group, u, null);
        }

        /// <summary>
        /// Create and add new payments
        /// </summary>
        /// <param name="price"></param>
        /// <param name="currency"></param>
        /// <param name="date"></param>
        /// <param name="purpose"></param>
        /// <param name="paid"></param>
        /// <param name="group"></param>
        /// <param name="user"></param>
        /// <returns>Added payment</returns>
        public Payment AddPayment(int price, PaymentCurrency currency, DateTime date, string purpose, bool paid, PaymentGroup group, User user)
        {
            return AddPayment(price, currency, date, purpose, paid, group, user, null);
        }

        /// <summary>
        /// Create and add new payment with note
        /// </summary>
        /// <param name="price"></param>
        /// <param name="currency"></param>
        /// <param name="date"></param>
        /// <param name="purpose"></param>
        /// <param name="paid"></param>
        /// <param name="group"></param>
        /// <param name="user"></param>
        /// <param name="note"></param>
        /// <returns>Added payment</returns>
        public Payment AddPayment(int price, PaymentCurrency currency, DateTime date, string purpose, bool paid, PaymentGroup group, User user, string note)
        {
            return AddPayment(new Payment(user)
            {
                Price = price,
                Currency = currency,
                Date = date,
                Purpose = purpose,
                Paid = paid,
                Group = group,
                Note = note
            });
        }

        /// <summary>
        /// Retrieve all payments
        /// </summary>
        /// <returns>List of payments</returns>
        public List<Payment> GetAllPayments()
        {
            return _context.Payments.ToList();
        }

        /// <summary>
        /// Get payment by id (primary key)
        /// </summary>
        /// <param name="id">Primary key</param>
        /// <returns>Payment</returns>
        public Payment GetPayment(int id)
        {
            return _context.Payments.FirstOrDefault(i => i.PaymentId == id);
        }

        /// <summary>
        /// Update payment
        /// </summary>
        /// <param name="p">Payment for update</param>
        /// <returns>Updated payment</returns>
        public Payment UpdatePayment(Payment p)
        {
            _context.Payments.Add(p);
            _context.SaveChanges();
            return p;
        }
 
        /// <summary>
        /// Delete payment by ID (primary key)
        /// </summary>
        /// <param name="id">Primary key</param>
        /// <returns>Deleted payment</returns>
        public Payment DeletePayment(int id)
        {
            Payment p = GetPayment(id);
            if (p != null)
            {
                Payment deleted = _context.Payments.Remove(p);
                _context.SaveChanges();
                return deleted;
            }
            else
            {
                throw new IndexOutOfRangeException("Payment with ID " + id + " don't exist!");
            }
        }

        /// <summary>
        /// Delete payment
        /// </summary>
        /// <param name="p">Payment to delete</param>
        /// <returns>Deleted payment</returns>
        public Payment DeletePayment(Payment p)
        {
            Payment deleted = _context.Payments.Remove(p);
            _context.SaveChanges();
            return deleted;
        }
    }
}
