﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PersonalFinance
{
    /// <summary>
    /// Service for communication to database
    /// </summary>
    public class ScheduledItemService
    {
        private EntityContext _context;
        public ScheduledItemService(EntityContext context)
        {
            _context = context;
        }

        /// <summary>
        /// Add ScheduledItem
        /// </summary>
        /// <param name="p">ScheduledItem</param>
        /// <returns>Added ScheduledItem</returns>
        public ScheduledItem AddScheduledItem(ScheduledItem p)
        {
            _context.ScheduledItems.Add(p);
            _context.SaveChanges();
            return p;
        }

        /// <summary>
        /// Add payment to scheduling
        /// </summary>
        /// <param name="payment">Payment to scheduling</param>
        /// <param name="freq">Frequency of scheduling</param>
        /// <returns>ScheduledItem</returns>
        public ScheduledItem AddPayment(Payment payment, Frequency freq)
        {
            return AddPayment(payment, freq, DateTime.Now, null);
        }

        /// <summary>
        /// Add payment to scheduling
        /// </summary>
        /// <param name="payment">Payment to scheduling</param>
        /// <param name="freq">Frequency of scheduling</param>
        /// <param name="lastUsed"></param>
        /// <returns>ScheduledItem</returns>
        public ScheduledItem AddPayment(Payment payment, Frequency freq, DateTime lastUsed)
        {
            return AddPayment(payment, freq, lastUsed, null);
        }

        /// <summary>
        /// Add payment to scheduling
        /// </summary>
        /// <param name="payment">Payment to scheduling</param>
        /// <param name="freq">Frequency of scheduling</param>
        /// <param name="lastUsed"></param>
        /// <param name="until">How long to be scheduled</param>
        /// <returns>ScheduledItem</returns>
        public ScheduledItem AddPayment(Payment payment, Frequency freq, DateTime lastUsed, DateTime? until)
        {
            return AddScheduledItem(new ScheduledItem()
            {
               Payment = payment,
               Frequency = freq,
               LastUse = lastUsed,
               Until = until
            });
        }

        /// <summary>
        /// Retrieve all ScheduledItems
        /// </summary>
        /// <returns></returns>
        public List<ScheduledItem> GetAllScheduledItems()
        {
            return _context.ScheduledItems.ToList();
        }

        /// <summary>
        /// Get ScheduledItems by ID (primary key)
        /// </summary>
        /// <param name="id">Primary key</param>
        /// <returns>ScheduledItems</returns>
        public ScheduledItem GetScheduledItem(int id)
        {
            return _context.ScheduledItems.FirstOrDefault(i => i.ScheduledItemId == id);
        }

        /// <summary>
        /// Update ScheduledItems
        /// </summary>
        /// <param name="p">ScheduledItems to update</param>
        /// <returns>Updated ScheduledItems</returns>
        public ScheduledItem UpdateScheduledItems(ScheduledItem p)
        {
            _context.ScheduledItems.Add(p);
            _context.SaveChanges();
            return p;
        }

        /// <summary>
        /// Delete ScheduledItems by ID  (Primary key)
        /// </summary>
        /// <param name="id">Primary key</param>
        /// <returns>Deleted ScheduledItems</returns>
        public ScheduledItem DeleteScheduledItem(int id)
        {
            ScheduledItem p = GetScheduledItem(id);
            if (p != null)
            {
                ScheduledItem deleted = _context.ScheduledItems.Remove(p);
                _context.SaveChanges();
                return deleted;
            }
            else
            {
                throw new IndexOutOfRangeException("ScheduledItem with ID " + id + " don't exist!");
            }
        }

        /// <summary>
        /// Delete ScheduledItems
        /// </summary>
        /// <param name="p">ScheduledItems to delete</param>
        /// <returns>Deleted ScheduledItems</returns>
        public ScheduledItem DeleteScheduledItem(ScheduledItem p)
        {
            ScheduledItem deleted = _context.ScheduledItems.Remove(p);
            _context.SaveChanges();
            return deleted;
        }
    }
}
