﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PersonalFinance
{
    /// <summary>
    /// Service for communication to database
    /// </summary>
    public class UserService
    {
        private EntityContext _context;
        public UserService(EntityContext context)
        {
            _context = context;
        }

        /// <summary>
        /// Add new user
        /// </summary>
        /// <param name="u">User</param>
        /// <returns>Added user</returns>
        public User AddUser(User u){
            _context.Users.Add(u);
            _context.SaveChanges();
            return u;
        }

        /// <summary>
        /// Create and add new user
        /// </summary>
        /// <param name="login"></param>
        /// <param name="name"></param>
        /// <param name="lastname"></param>
        /// <param name="password"></param>
        /// <param name="sex"></param>
        /// <returns>Added user</returns>
        public User AddUser(string login, string name, string lastname, string password, Sex sex)
        {
            return AddUser(new User(password) { Login = login, Name = name, Lastname = lastname, Sex = sex });
        }

        /// <summary>
        /// Retrieve all users
        /// </summary>
        /// <returns>List of users</returns>
        public List<User> GetAllUsers()
        {
            return _context.Users.ToList();
        }

        /// <summary>
        /// Get user by ID (Primary key)
        /// </summary>
        /// <param name="id">int Primary key</param>
        /// <returns>User</returns>
        public User GetUser(int id)
        {
            return _context.Users.FirstOrDefault(i => i.UserId == id);
        }
        /// <summary>
        /// Get user by ID (Primary key)
        /// </summary>
        /// <param name="login">string</param>
        /// <returns>User</returns>
        public User GetUser(string login)
        {
            return _context.Users.FirstOrDefault(i => i.Login == login);
        }

        /// <summary>
        /// Delete user by ID (primary key)
        /// </summary>
        /// <param name="id">int Primary key</param>
        /// <returns>Deleted user</returns>
        public User DeleteUser(int id)
        {
            User u = GetUser(id);
            if (u != null)
            {
                User deleted = _context.Users.Remove(u);
                _context.SaveChanges();
                return deleted;
            }
            else
            {
                throw new IndexOutOfRangeException("User with ID " + id + " don't exist!");
            }
        }

        /// <summary>
        /// Delete user
        /// </summary>
        /// <param name="u">User to delete</param>
        /// <returns>Deleted user</returns>
        public User DeleteUser(User u)
        {
            User deleted = _context.Users.Remove(u);
            _context.SaveChanges();
            return deleted;
        }

        /// <summary>
        /// Update user
        /// </summary>
        /// <param name="u">User to update</param>
        /// <returns>Updated user</returns>
        public User UpdateUser(User u)
        {
            _context.Users.Add(u);
            _context.SaveChanges();
            return u;
        }
    }
}
